xquery version "3.1";

import module namespace elastic="http://elastic.io" at "elastic.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "config.xqm";
import module namespace lesapi="http://sepia.io/esapi/legacy-api";
(:import module namespace console="http://exist-db.org/xquery/console";:)

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

declare option output:method "json";
declare option output:media-type "application/json";

declare variable $action := request:get-parameter('action', 'search');
declare variable $terms := request:get-parameter('terms', '');
declare variable $page := xs:integer(request:get-parameter('page', 1));
declare variable $per_page := xs:integer(request:get-parameter('per_page', 10));
declare variable $locale := request:get-parameter('locale', 'any');


(:
  Defines the endpoints for the json search api. Each function essentially
  queries an elasticsearch index for results with highlighting, pagination and
  locale filters.

  The interaction with elasticsearch (http requests) can be found in
  /modules/elastic.xqm.
:)

declare function local:search-editions-elastic() {
    let $filters := if ($locale eq 'any') then [] else [
        map {"term": map {"locale": $locale}}
    ]
    let $response := lesapi:search($elastic:uri, $elastic:prefix || "editions", map {
        "from": ($page - 1) * $per_page,
        "size": $per_page,
        "query": map {
            "bool": map {
                "must": [
                    map {
                        "query_string": map {
                            "query": $terms
                        }
                    }
                ],
                "filter": $filters
            }
        },
        "highlight": map {
            "fragment_size": 100,
            "require_field_match": fn:false(),
            "pre_tags": ['<span class="hl">'],
            "post_tags": ['</span>'],
            "type": "plain",
            "highlight_query": map {
                "query_string": map {
                    "query": $terms
                }
            },
            "fields": map {
              "name": map {"number_of_fragments": 0},
              "search_data_a": map {},
              "search_data_b": map {},
              "search_data_c": map {},
              "search_data_d": map {},
              "search_data_e": map {},
              "search_data_f": map {}
            }
        }
    })
    return $response
};

declare function local:search-commentaries-elastic() {
    let $filters := if ($locale eq 'any') then [] else [
        map {"term": map {"locale": $locale}}
    ]
    let $response := lesapi:search($elastic:uri, $elastic:prefix || "commentaries", map {
        "from": ($page - 1) * $per_page,
        "size": $per_page,
        "query": map {
            "bool": map {
                "must": [
                    map {
                        "query_string": map {
                            "query": $terms
                        }
                    }
                ],
                "filter": $filters
            }
        },
        "highlight": map {
            "fragment_size": 100,
            "require_field_match": fn:false(),
            "pre_tags": ['<span class="hl">'],
            "post_tags": ['</span>'],
            "type": "plain",
            "highlight_query": map {
                "query_string": map {
                    "query": $terms
                }
            },
            "fields": map {
              "name": map {"number_of_fragments": 0},
              "search_data": map {}
            }
        }
    })
    return $response
};



declare function local:search-pages-elastic() {
    let $filters := if ($locale eq 'any') then [] else [
        map {"term": map {"locale": $locale}}
    ]
    let $response := lesapi:search($elastic:uri, $elastic:prefix || "wiki_pages", map {
        "from": ($page - 1) * $per_page,
        "size": $per_page,
        "query": map {
            "bool": map {
                "must": [
                    map {
                        "query_string": map {
                            "query": $terms
                        }
                    }
                ],
                "filter": $filters
            }
        },
        "highlight": map {
            "fragment_size": 100,
            "require_field_match": fn:false(),
            "pre_tags": ['<span class="hl">'],
            "post_tags": ['</span>'],
            "type": "plain",
            "highlight_query":  map {
                "query_string": map {
                    "query": $terms
                }
            },
            "fields": map {
              "search_data": map {}
            }
        }
    })
    return $response
};

declare function local:search-register-elastic($type) {
    let $fields := if ($locale eq 'de') then
        ['search_data.de']
    else if ($locale eq 'fr') then
        ['search_data.fr']
    else
        ['search_data.de', 'search_data.fr']
    let $response := lesapi:search($elastic:uri, $elastic:prefix || $type, map {
        "from": ($page - 1) * $per_page,
        "size": $per_page,
        "query": map {
            "query_string": map {
                "query": $terms,
                "fields": $fields
            }
        },
        "highlight": map {
            "fragment_size": 100,
            "require_field_match": fn:false(),
            "pre_tags": ['<span class="hl">'],
            "post_tags": ['</span>'],
            "type": "plain",
            "fields": map {
                "name.de": map {"number_of_fragments": 0},
                "name.fr": map {"number_of_fragments": 0},
                "search_data.de": map {},
                "search_data.fr": map {}
            }
        }
    })
    return $response
};

declare function local:error($message as xs:string, $status as xs:integer) {
    let $x := response:set-status-code($status)
    return
        map {'message': $message}
};

declare function local:route() {
    if ($action eq 'search-people') then
        local:search-register-elastic('people')
    else if ($action eq 'search-works') then
        local:search-register-elastic('works')
    else if ($action eq 'search-editions') then
        local:search-editions-elastic()
    else if ($action eq 'search-commentaries') then
        local:search-commentaries-elastic()
    else if ($action eq 'search-pages') then
        local:search-pages-elastic()
    else
        local:error('unknown action', 400)
};

local:route()
